# -*- coding: utf-8 -*-

import model
import containers
import model.cc


def create(parent):
    """create the stl library"""
    kwargs = {}
    kwargs['parent'] = parent
    kwargs['name'] = 'stl'

    library = model.Library(**kwargs)

    kwargs['parent'] = library
    kwargs['name'] = 'std'
    namespace = model.cc.Namespace(**kwargs)

    containers.create(namespace)


"""Subclass of NewTask, which is generated by wxFormBuilder."""

import wx
import wxx
import app.resources as rc
import model.tasks as tasks
from activity.tasks.ui import ui


# Implementing NewTask
class TaskDialog(ui.NewTask):
    """This dialog allows to configure the task properties.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        self._container = container
        self.origin = None
        super(TaskDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("glass_clock"))
        self.SetIcon(icon)
        self.m_staticText6.SetLabelText(wx.DateTime.Now().Format('%Y-%m-%d %H:%M:%S'))
        self.m_staticText7.SetLabelText('--/--/---- --:--:--')
        self.m_staticText72.SetLabelText('--/--/---- --:--:--')
        # The kind of parent determines the kind of task
        if type(self._container) is tasks.PendingTasks:
            self.m_choice1.SetSelection(0)
        elif type(self._container) is tasks.CurrentTasks:
            self.m_choice1.SetSelection(1)
        elif type(self._container) is tasks.FinishedTasks:
            self.m_choice1.SetSelection(2)
        self.m_choice1.Enable(False)

    def Validate(self):
        """Validate the dialog and returns ok or not ok"""
        # get name
        self._name = self.m_textCtrl2.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Task Short description must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        # get status
        i = self.m_choice1.GetSelection()
        self._status = self.m_choice1.GetString(i)
        # get priority
        i = self.m_choice3.GetSelection()
        self._priority = self.m_choice3.GetString(i)
        # get type
        i = self.m_choice2.GetSelection()
        self._taskType = self.m_choice2.GetString(i)
        # get note
        self._note = self.m_richText3.GetValue()
        # get dates
        self._dateCreated = self.m_staticText6.GetLabelText()
        self._dateBegin = self.m_staticText7.GetLabelText()
        self._dateEnd = self.m_staticText72.GetLabelText()
        return True

    def CopyAttributes(self, task):
        """Transfer attributes to task"""
        task._name = self._name
        task.note = self._note
        task._status = self._status
        task._priority = self._priority
        task._taskType = self._taskType
        task._dateCreated = self._dateCreated
        task._dateBegin = self._dateBegin
        task._dateEnd = self._dateEnd

    def SetAttributes(self, task):
        """Transfer attributes from existing task"""
        self._container = task.parent
        self._name = task._name
        self._note = task._note
        self._status = task._status
        self._priority = task._priority
        self._taskType = task._taskType
        self._dateCreated = task._dateCreated
        self._dateBegin = task._dateBegin
        self._dateEnd = task._dateEnd
        #
        i = self.m_choice1.FindString(self._status)
        if i != wx.NOT_FOUND:
            self.m_choice1.SetSelection(i)
        #
        i = self.m_choice3.FindString(self._priority)
        if i != wx.NOT_FOUND:
            self.m_choice3.SetSelection(i)
        #
        i = self.m_choice2.FindString(self._taskType)
        if i != wx.NOT_FOUND:
            self.m_choice2.SetSelection(i)
        #
        self.m_textCtrl2.SetValue(self._name)
        self.m_richText3.SetValue(self._note)
        self.m_staticText6.SetLabelText(self._dateCreated)
        self.m_staticText7.SetLabelText(self._dateBegin)
        self.m_staticText72.SetLabelText(self._dateEnd)
        self.SetTitle("Edit task")

    def get_kwargs(self):
        """return arguments suitable for object instance"""
        return {'parent': self._container, 'name': self._name,
            'note': self._note, 'status': self._status,
            'priority': self._priority, 'type': self._taskType,
            'dateCreated': self._dateCreated, 'dateBegin': self._dateBegin,
            'dateEnd': self._dateEnd,
            }

    # Handlers for NewTask events.
    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:31:28 2013

@author: mel
"""

import model
import tran
import app.resources as rc
from Argument import Argument


class ArgsArgument(Argument):
    """Implements argument representation"""

    context_container = True

    #visual methods
    @tran.TransactionalMethod('move argument {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_argument_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0  # trick for insert as first child
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        kwargs['name'] = 'args'
        kwargs['default'] = ''
        super(ArgsArgument, self).__init__(**kwargs)
        container = self.outer_class or self.outer_module
        container._lastSrcTime = None
        container._lastHdrTime = None

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('py_args')

    @property
    def label(self):
        """Get tree label"""
        return '*{self._name}'.format(self=self)




"""Subclass of BuildTools, which is generated by wxFormBuilder."""

import wx
import wxx
import app


# Implementing BuildTools
class BuildTools(app.BuildTools):
    """
    This dialog is used for setup of build tools.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Dialog initialization"""
        super(BuildTools, self).__init__(parent)
        import pane
        self._toolsPane = pane.BuildBinaries(self)
        if wx.__version__ >= '3.0.0.0':
            self.m_auinotebook4.AddPage(self._toolsPane, u"Binaries", False, wx.wx.NOT_FOUND)
        else:
            self.m_auinotebook4.AddPage(self._toolsPane, u"Binaries", False, wx.NullBitmap)





# -*- coding: utf-8 -*-


import models
import files
import tasks
import git
import targets

ALL = [
    models.ui.view.ModelsView,
    files.ui.view.FilesView,
    tasks.ui.view.TasksView,
    git.ui.view.GitView,
    targets.ui.view.TargetsView,
    ]

INDEX = dict([(x.name, x) for x in ALL])


# -*- coding: utf-8 -*-

"""Delimmited ast : ast nodes produced ast.parse(text)
only contains lineno and offset values, that represents
the start of ast node in the text being pased.
The astd.parse will return a modified ast tree with
to_line and to_offset values"""

import ast


def parse(text):
    """parse ast delimmited text"""
    try:
        tree = ast.parse(text)
        l = ['0'] + text.splitlines()
        __process_node(tree, l)
        return tree
    except Exception as inst:
        import traceback
        import sys
        traceback.print_exc(file=sys.stdout)
        print type(inst)     # the exception instance
        print inst.args      # arguments stored in .args
        print inst
        raise inst


def __cutoff_node(node, to_lineno, to_col_offset):
    """Cut node lengths"""
    if hasattr(node, 'lineno'):
        if hasattr(node, 'to_lineno'):
            if node.to_lineno < to_lineno:
                return
            if node.to_lineno == to_lineno and node.to_col_offset <= to_col_offset:
                return
        setattr(node, 'to_lineno', to_lineno)
        setattr(node, 'to_col_offset', to_col_offset)
    for cur in ast.iter_child_nodes(node):
        __cutoff_node(cur, to_lineno, to_col_offset)


def __process_node(node, l):
    """Compute offsets"""
    prev = None
    for cur in ast.iter_child_nodes(node):
        if hasattr(cur, 'lineno'):
            __process_node(cur, l)
        if type(cur) is ast.Str:
            s = cur.s
            if s:
                if s[0] == '"':
                    s = s[1:]
                if s and s[-1] == '"':
                    s = s[:-1]
                v = (s + ' ').splitlines()
                v[-1] = v[-1][:-1]
                if len(v) > 1 and not v[0]:
                    del v[0]
            else:
                v = ['']
            n = len(v)
            if cur.col_offset >= 0:
                left_lin = cur.lineno
                left_qpos = l[left_lin].find('"""', cur.col_offset)
                if left_qpos < 0:
                    left_qpos = l[left_lin].find("'''", cur.col_offset)
                if left_qpos < 0:
                    dpos = l[left_lin].find('"', cur.col_offset)
                    spos = l[left_lin].rfind("'", cur.col_offset)
                    if dpos > spos:
                        left_qpos = dpos
                    else:
                        left_qpos = spos
                    left_pos = left_qpos + 1
                else:
                    left_pos = left_qpos + 3
            else:
                left_lin = cur.lineno - n + 1
                if v[0]:
                    left_pos = l[left_lin].rfind(v[0])
                else:
                    left_pos = 0
                while left_pos == 0:
                    left_lin = left_lin - 1
                    left_pos = len(l[left_lin])
                left_qpos = l[left_lin].rfind('"""')
                if left_qpos < 0:
                    left_qpos = l[left_lin].rfind("'''")
                if left_qpos < 0:
                    dpos = l[left_lin].rfind('"')
                    spos = l[left_lin].rfind("'")
                    if dpos > spos:
                        left_qpos = dpos
                    else:
                        left_qpos = spos
                    left_pos = left_qpos + 1
                else:
                    left_pos = left_qpos + 3
            right_lin = cur.lineno
            if v[-1]:
                if cur.col_offset > -1:
                    right_pos = l[right_lin].find(v[-1], cur.col_offset) + len(v[-1])
                else:
                    right_pos = l[right_lin].find(v[-1]) + len(v[-1])
            else:
                right_pos = 0
            while len(l[right_lin]) == 0:
                right_lin = right_lin + 1
                right_pos = 0
            right_qpos = l[right_lin].find('"""', right_pos) + 3
            if right_qpos < 3:
                right_qpos = l[right_lin].find("'''", right_pos) + 3
            if right_qpos < 3:
                dpos = l[right_lin].find('"')
                spos = l[right_lin].find("'")
                if dpos > 0 and dpos < spos:
                    right_qpos = dpos
                else:
                    right_qpos = spos
                right_pos = right_qpos - 1
            else:
                right_pos = right_qpos - 3

            setattr(cur, 'to_lineno', right_lin)
            setattr(cur, 'to_col_offset', right_pos)
            cur.lineno = left_lin
            cur.col_offset = left_pos
            if type(node) is ast.Expr:
                node.lineno = left_lin
                node.col_offset = left_qpos
                node.to_lineno = right_lin
                node.to_col_offset = right_qpos

        if hasattr(cur, 'lineno'):
            if prev:
                #find previous non empty
                lin = cur.lineno
                pos = cur.col_offset
                while True:
                    while pos and l[lin][pos - 1:pos].isspace():
                        pos = pos - 1
                    if pos == 0:
                        lin = lin - 1
                        while len(l[lin].strip()) == 0 or l[lin].strip()[0] == '#':
                            lin = lin - 1
                        pos = len(l[lin])
                    else:
                        __cutoff_node(prev, lin, pos)
                        break
        prev = cur
    if prev:
        if hasattr(node, 'to_lineno'):
            lin = node.to_lineno
            pos = node.to_col_offset
        else:
            lin = len(l) - 1
            pos = len(l[lin])
        while True:
            while pos and l[lin][pos - 1:pos].isspace():
                pos = pos - 1
            if pos == 0:
                lin = lin - 1
                pos = len(l[lin])
            else:
                __cutoff_node(prev, lin, pos)
                break




# -*- coding: utf-8 -*-
import wx
import app
from  ctx import theContext as context


class AppSplashScreen(wx.SplashScreen):
    """
    Create a splash screen widget.
    """
    def __init__(self, parent=None):
        """Initialize splash screen"""
        aBitmap = wx.Image(name="app/res/beatle.jpg").ConvertToBitmap()
        splashStyle = wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_TIMEOUT
        splashDuration = 2000  # milliseconds
        # Call the constructor with the above arguments in exactly the
        # following order.
        wx.SplashScreen.__init__(self, aBitmap, splashStyle,
                                 splashDuration, parent)
        self.Bind(wx.EVT_CLOSE, self.OnExit)
        wx.YieldIfNeeded()

    def OnExit(self, evt):
        """Continue process"""
        self.Hide()
        app.mainWindow(self)
        context.frame.Show()
        wx.GetApp().SetTopWindow(context.frame)
        # The program will freeze without this line.
        evt.Skip()  # Make sure the default handler runs too...



# -*- coding: utf-8 -*-
import wx

from _expr import _expr
from _module import _module
from _import import _import
from _str import _str
from _alias import _alias
from _assign import _assign
from _name import _name
from _binop import _binop
from _store import _store
from _load import _load
from _attribute import _attribute
from _add import _add
from _num import _num
from _classdef import _classdef
from _functiondef import _functiondef
from _arguments import _arguments
from _param import _param
from _call import _call
from _dict import _dict
from _return import _return
from _listcomp import _listcomp
from _tuple import _tuple
from _comprehension import _comprehension
from _global import _global
from _for import _for
from _keyword import _keyword
from _if import _if
from _bool import _bool
from _and import _and
from _compare import _compare
from _in import _in
from _subscript import _subscript
from _index import _index
from _unary import _unary
from _not import _not
from _eq import _eq
from _not_eq import _not_eq
from _delete import _delete
from _del import _del
from _sub import _sub
from _not_in import _not_in
from _gte import _gte
from _lte import _lte
from _lt import _lt
from _gt import _gt
from _is import _is
from _is_not import _is_not
from _floor_div import _floor_div
from _continue import _continue
from _try import _try
from _or import _or
from _except import _except
from _print import _print
from _list import _list

_xpm = [_expr, _module, _import, _str, _alias, _assign, _name, _binop, _store, _load,
        _attribute, _add, _num, _classdef, _functiondef, _arguments, _param, _call,
        _dict, _return, _tuple, _listcomp, _comprehension, _global, _for, _keyword,
        _if, _bool, _and, _compare, _in, _subscript, _index, _unary, _not,
        _eq, _not_eq, _delete, _del, _sub, _not_in, _gte, _lte, _lt, _gt, _is,
        _is_not, _floor_div, _continue, _try, _or, _except, _print, _list]

_bitmapList = []


def CreateBitmapList():
    """Crea la lista de bitmaps"""
    global _bitmapList
    global _xpm
    for xpm in _xpm:
        bmp = wx.BitmapFromXPMData(xpm)
        _bitmapList.append(bmp)


def GetBitmapIndex(name, access=None):
    """Return the bitmap index"""
    try:
        base_index = _xpm.index(eval('_{}'.format(name)))
    except:
        return wx.NOT_FOUND
    return base_index


def GetBitmap(name):
    """Get the specific bitmap"""
    index = GetBitmapIndex(name)
    if index == wx.NOT_FOUND:
        raise KeyError('bitmap {} not found'.format(name))
        return None
    global _bitmapList
    if not len(_bitmapList):
        CreateBitmapList()
    return _bitmapList[index]


def GetBitmapImageList():
    """Returns the bitmap image list"""
    global _bitmapList
    if len(_bitmapList) == 0:
        CreateBitmapList()
    imglist = wx.ImageList(16, 16, True, len(_bitmapList))
    for bmp in _bitmapList:
        imglist.Add(bmp)
    return imglist



# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""


import model
import model.decorator as ctx
import app.resources as rc
from tran import TransactionStack



class Enum(model.TComponent):
    """Implements member data"""
    context_container = True

    def __init__(self, **kwargs):
        """Initialization. The items argument is an optional list
        whose elements are a pair (label, value). """
        self._items = kwargs.get('items', [])
        self._access = kwargs.get('access', 'public')
        super(Enum, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Enum, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['items'] = self._items
        kwargs['access'] = self._access
        kwargs.update(super(Enum, self).get_kwargs())
        return kwargs

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("enum", self._access)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Enum, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(Enum, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    @property
    def scoped(self):
        """Get the scope"""
        return '{self.parent.scope}{self._name}'.format(self=self)

    @property
    def scope(self):
        """Get the scope"""
        return '{scoped}::'.format(scoped=self.scoped)

    @property
    def label(self):
        """Get tree label"""
        return 'enum {self._name}'.format(self=self)

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the declaration of the enum using writer"""
        cls = self.inner_class
        if cls is not None:
            cls._ensure_access(pf, self._access)
        #ok, write it
        self.WriteComment(pf)
        pf.writeln('enum {self._name}'.format(self=self))
        pf.openbrace()
        for i in range(0, len(self._items) - 1):
            item = self._items[i]
            k = item[0].strip()
            s = item[1].strip()
            if len(s):
                pf.writeln("{0}={1},".format(k, s))
            else:
                pf.writeln("{0},".format(k))
        item = self._items[-1]
        k = item[0].strip()
        s = item[1].strip()
        if len(s):
            pf.writeln("{0}={1}".format(k, s))
        else:
            pf.writeln("{0}".format(k))
        pf.closebrace(";")





# -*- coding: utf-8 -*-
"""The mission of this module is analyzing python code and
recreate hist structure"""

import ast
from analytic import astd
import model


class pyparse(object):
    """Class for parsing a python text"""

    project = None

    @classmethod
    def set_project(this, project):
        """Sets the project"""
        pyparse.project = project

    def __init__(self, container, text):
        """Initializes the parser and do the work"""
        super(pyparse, self).__init__()
        self._outlines = []  # the set of lines to remove
        self._container = container
        self._text = text
        self._set_handlers()
        self._status = self.analize()

    def __imports_folder(self, parent):
        """Access or create to import folder"""
        s = parent[model.py.ImportsFolder]
        if len(s):
            return s[0]
        kwargs = {'parent': parent}
        return model.py.ImportsFolder(**kwargs)

    def inline(self):
        """This method returns the text without what whas processed"""
        l = ['0'] + self._text.splitlines()
        u = [l[i] for i in range(1, len(l)) if i not in self._outlines]
        return '\n'.join(u)

    def _set_handlers(self):
        """The ast tree is composed by several types of nodes
        each one representing a kind of python expression.
        This method setup the handlers for each one type of
        nodes"""
        self._handler = {
            ast.Expr: self.process_expr,
            ast.Import: self.process_import,
            ast.ImportFrom: self.process_import_from,
            ast.ClassDef: self.process_class_def,
            ast.FunctionDef: self.process_function_def,
            ast.Assign: self.process_var
            }

        self._text_handler = {
            ast.Expr: self._text_expr,
            ast.Attribute: self._text_attribute,
            ast.Name: self._text_name,
            ast.Str: self._text_str,
            ast.Num: self._num_str,
            ast.BinOp: self._binop_str,
            ast.Add: self._add_str,
            ast.Sub: self._sub_str,
            ast.Mult: self._mult_str,
            ast.Div: self._div_str,
            ast.FloorDiv: self._floordiv_str,
            ast.Mod: self._mod_str,
            ast.Pow: self._pow_str,
            ast.LShift: self._lshift_str,
            ast.RShift: self._rshift_str,
            ast.BitOr: self._bitor_str,
            ast.BitXor: self._bitxor_str,
            ast.BitAnd: self._and_str,
            ast.keyword: self._keyword_str,
            ast.Dict: self._dict_str,
            ast.Tuple: self._tuple_str,
            ast.List: self._list_str,
            ast.Subscript: self._subscript_str,
            ast.Index: self._index_str,
            ast.Call: self._call_str,
            ast.UnaryOp: self._unary_str,
            }

    def call_handler(self, parent, node):
        """Do the call"""
        return self._handler.get(type(node), self.process_none)(parent, node)

    def process_node(self, parent, node):
        """This is the root of process analysing"""
        for child_node in ast.iter_child_nodes(node):
            self.call_handler(parent, child_node)
        return True

    def text_node(self, node):
        """This represents the text converter"""
        self.outline(node)
        return self._text_handler.get(type(node), self.text_none)(node)

    def text_none(self, node):
        """This represents the missing handler case"""
        print '\nmissing handler for {t}'.format(t=type(node))
        return ''

    def process_none(self, parent, node):
        """Do an empty process for unknown nodes"""
        return True

    def process_expr(self, parent, node):
        """Add comment to parent"""
        self.outline(node)
        parent.note = self.text_node(node.value).strip()[1:-1]

    def process_import(self, parent, node):
        """Process a import node"""
        if len(node.names):
            kwargs = {'parent': self.__imports_folder(parent)}
            for n in node.names:
                kwargs['name'] = n.name
                kwargs['as'] = n.asname
                model.py.Import(**kwargs)
        self.outline(node)
        return True

    def process_import_from(self, parent, node):
        """Process a import from node"""
        if len(node.names):
            kwargs = {
                'parent': self.__imports_folder(parent),
                'name': node.module
            }
            for n in node.names:
                kwargs['from'] = n.name
                kwargs['as'] = n.asname
                model.py.Import(**kwargs)
        self.outline(node)
        return True

    def process_class_def(self, parent, node):
        """Process a class definition"""
        kwargs = {
            'parent': parent,
            'name': node.name,
            'raw': True,
            'note': ast.get_docstring(node, False) or ''
            }
        if not kwargs['note']:
            i = 0
        else:
            i = 1
        # the base classes must be resolved later
        cls = model.py.Class(**kwargs)
        # annotate bases for later resolution
        if node.bases:
            if pyparse.project:
                kwargs['parent'] = cls
                kwargs['raw'] = True
                for base in node.bases:
                    kwargs['ancestor'] = None
                    kwargs['name'] = self.text_node(base)
                    model.py.Inheritance(**kwargs)
            else:
                cls._bases = node.bases
        for j in range(i, len(node.body)):
            self.call_handler(cls, node.body[j])
        self.outline(node)
        return True

    def process_var(self, parent, node):
        """Process a single global asign"""
        kwargs = {'parent': parent, 'value': self.text_node(node.value)}
        if type(parent) is model.py.Class:
            theClass = model.py.MemberData
        else:
            theClass = model.py.Data
        for v in node.targets:
            if type(v) is not ast.Name:
                # we dont implement this
                continue
            kwargs['name'] = v.id
            self.outline(v)
            theClass(**kwargs)
        return True

    def process_decorator(self, parent, node):
        """Process a decorator node"""
        kwargs = {'parent': parent}
        if type(node) is ast.Name:
            kwargs['name'] = '{node.id}'.format(node=node)
        if type(node) is ast.Call:
            kwargs['call'] = True
            kwargs['name'] = self.text_node(node.func)
        else:
            kwargs['name'] = self.text_node(node)
        deco = model.py.Decorator(**kwargs)
        if type(node) is ast.Call:
            for arg in node.args:
                self.process_arg_def(deco, arg, context='value')
        return True

    def _text_expr(self, node):
        """process a ast.Expr"""
        return self.text_node(node.value)

    def _text_attribute(self, node):
        """process a ast.Attribute"""
        attr = node.attr
        if node.value:
            value = self.text_node(node.value)
            return '{value}.{attr}'.format(value=value, attr=attr)
        return attr

    def _text_name(self, node):
        """process ast.Name"""
        return node.id

    def _text_str(self, node):
        """process a ast.Str"""
        if '"' in node.s:
            return "'{node.s}'".format(node=node)
        return '"{node.s}"'.format(node=node)

    def _num_str(self, node):
        """process a ast.Num"""
        return "{node.n}".format(node=node)

    def _binop_str(self, node):
        """process ast.BinOp"""
        left = self.text_node(node.left)
        right = self.text_node(node.right)
        operand = self.text_node(node.op)
        return '{left} {operand} {right}'.format(
            left=left, right=right, operand=operand)

    def _add_str(self, node):
        """process ast.Add"""
        return '+'

    def _sub_str(self, node):
        """process ast.Sub"""
        return '-'

    def _mult_str(self, node):
        """process ast.Mult"""
        return '*'

    def _div_str(self, node):
        """process ast.Div"""
        return '/'

    def _floordiv_str(self, node):
        """process ast.FloorDiv"""
        return '//'

    def _mod_str(self, node):
        """process ast.Mod"""
        return '%'

    def _pow_str(self, node):
        """process ast.Pow"""
        return '**'

    def _lshift_str(self, node):
        """process ast.LShift"""
        return '<<'

    def _rshift_str(self, node):
        """process ast.RShift"""
        return '>>'

    def _bitor_str(self, node):
        """process ast.BitOr"""
        return '|'

    def _bitxor_str(self, node):
        """process ast.BitXor"""
        return '^'

    def _and_str(self, node):
        """process ast.BitAnd"""
        return '&'

    def _keyword_str(self, node):
        """process ast.keyword"""
        value = self.text_node(node.value)
        arg = node.arg
        return '{arg}={value}'.format(arg=arg, value=value)

    def _dict_str(self, node):
        """process ast.Dict"""
        assert(len(node.keys) == len(node.values))
        return '{{{cnt}}}'.format(cnt=', '.join(['{k}: {v}'.format(
            k=self.text_node(node.keys[i]),
            v=self.text_node(node.values[i])) for i in range(0, len(node.keys))]))

    def _tuple_str(self, node):
        """process ast.Tuple"""
        return '({cnt})'.format(cnt=', '.join([self.text_node(k) for k in node.elts]))

    def _list_str(self, node):
        """process ast.List"""
        return '[{cnt}]'.format(cnt=', '.join([self.text_node(k) for k in node.elts]))

    def _subscript_str(self, node):
        """process ast.Subscript"""
        name = self.text_node(node.name)
        _slice = self.text_node(node.slice)
        return '{name}[{slice}]'.format(name=name, slice=_slice)

    def _index_str(self, node):
        """process ast.Index"""
        return self.text_node(node.value)

    def _call_str(self, node):
        """process ast.Call"""
        args = [self.text_node(x) for x in node.args or []]
        args.extend([self.text_node(x) for x in node.keywords])
        if node.starargs:
            args.append('*{}'.format(self.text_node(node.starargs)))
        if node.kwargs:
            args.append('**{}'.format(self.text_node(node.kwargs)))
        func = self.text_node(node.func)
        args = ', '.join(args)
        return '{func}({args})'.format(func=func, args=args)

    def _unary_str(self, node):
        """process ast.UnaryOp"""
        if type(node.op) == ast.Invert:
            return '~{}'.format(self.text_node(node.operand))
        elif type(node.op) == ast.Not:
            return 'not {}'.format(self.text_node(node.operand))
        elif type(node.op) == ast.UAdd:
            return '+{}'.format(self.text_node(node.operand))
        elif type(node.op) == ast.USub:
            return '-{}'.format(self.text_node(node.operand))
        else:
            return self.text_none(node)

    def process_arg_def(self, parent, node, **kwargs):
        """Process an argument node"""
        self.outline(node)
        kwargs = {
            'parent': parent,
            'name': self.text_node(node),
            'default': kwargs.get('default', ''),
            'context': kwargs.get('context', 'declare')
            }
        model.py.Argument(**kwargs)
        return True

    def process_function_def(self, parent, node):
        """Process a function definition"""
        kwargs = {
            'parent': parent,
            'name': node.name,
            'raw': True
            }
        for deco in node.decorator_list:
            self.outline(deco)
        if node.lineno not in self._outlines:
            self._outlines.append(node.lineno)
        # get the comment if any
        line = None
        if len(node.body) > 0:
            line = node.body[0].lineno
            col = node.body[0].col_offset
            if type(node.body[0]) is ast.Expr:
                if type(node.body[0].value) is ast.Str:
                    self.outline(node.body[0])
                    line = node.body[0].to_lineno + 1
                    kwargs['note'] = node.body[0].value.s
        #line = max(self._outlines) + 1
        if type(parent) is model.py.Class:
            """Se trata de una funcion miembro"""
            if node.name == '__init__':
                function = model.py.InitMethod(**kwargs)
            else:
                function = model.py.MemberMethod(**kwargs)
        else:
            function = model.py.Function(**kwargs)
        #Ok, add decorators
        for deco in node.decorator_list:
            self.process_decorator(function, deco)
        #Ok, add arguments
        kwargs['parent'] = function
        kwargs['note'] = ''
        v = node.args.defaults
        i = len(v)
        k = len(node.args.args)
        for arg in node.args.args:
            # this is valid for python3. For the moment, we
            # work in python 2
            #self.call_handler(function, arg)
            if k > i:
                self.process_arg_def(function, arg)
            else:
                self.process_arg_def(function, arg, default=self.text_node(v[-i]))
                i = i - 1
            k = k - 1
        if node.args.vararg is not None:
            model.py.ArgsArgument(**kwargs)
        if node.args.kwarg is not None:
            model.py.KwArgsArgument(**kwargs)
        #push all lines
        self.outline(node)
        #now, extract content from lines
        if line:
            code = [0] + self._text.splitlines()
            code = [code[i] for i in range(line, node.to_lineno + 1)]
            lcode = len(code)
            if lcode:
                try:
                    i = (n for n in range(0, lcode) if len(code[n].strip()) > 0).next()
                    test = code[i]
                    extra = len(test) - len(test.lstrip())
                    fcode = []
                    if extra:
                        # filter comments that will be addeded and cutted wrong
                        for i in range(0, lcode):
                            s = code[i]
                            if len(s.strip()) and len(s[:extra].strip()):
                                continue
                            else:
                                fcode.append(s)
                        lcode = len(fcode)
                        code = fcode
                        code = [code[i][extra:] for i in range(0, lcode)]
                        col -= extra
                        if col < 0:
                            col = 0
                except:
                    pass
            #en la primera linea del codigo hemos de tratar de todas maneras la columna
            #por si se trata de una expresion inline
            if code:
                try:
                    code[0] = code[0][col:]
                    function._content = '\n'.join(code)
                except:
                    print "col bug"
                #for some reason, some nodes are not simply traveled as childs
        return True

    def outline(self, node):
        """This method adds the lines represented by the node
        and his childs to the _outlines"""
        if hasattr(node, 'lineno'):
            s = node.lineno
            if hasattr(node, 'to_lineno'):
                t = node.to_lineno
                if s and t:
                    for i in range(s, t + 1):
                        if i not in self._outlines:
                            self._outlines.append(i)
                    return
            if s is not None and s not in self._outlines:
                self._outlines.append(s)
        for child_node in ast.iter_child_nodes(node):
            self.outline(child_node)

    def analize(self):
        """This method analizes the code"""
        import traceback
        import sys
        try:
            tree = astd.parse(self._text)
            return self.process_node(self._container, tree)
        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            return False






"""Subclass of MethodPane, which is generated by wxFormBuilder."""

import wx
import wx.stc
import app
from ctx import theContext as context
import tran
from app.ui import pane
from activity.models.ui import ui as ui
from activity.models.ui.dlg.cc import NavigatorDialog
from model import cc
from analytic.cppparse import gather_vars

CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing MethodPane
class MethodPane(ui.MethodPane):
    """Implements method pane editor"""
    # command ids:
    _saveCode = wx.Window.NewControlId()
    _undoCode = wx.Window.NewControlId()
    _redoCode = wx.Window.NewControlId()
    _copyCode = wx.Window.NewControlId()
    _cutCode = wx.Window.NewControlId()
    _pasteCode = wx.Window.NewControlId()
    _deleteCode = wx.Window.NewControlId()
    _toggleFull = wx.Window.NewControlId()
    _navigator = wx.Window.NewControlId()

    def _create_toolbars(self):
        """Create custom toolbar"""
        self.m_localToolbar = wx.aui.AuiToolBar(self, wx.ID_ANY, wx.DefaultPosition,
            wx.DefaultSize, wx.aui.AUI_TB_HORZ_LAYOUT)
        self.m_save = self.m_localToolbar.AddTool(self._saveCode,
            u"save code", wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_TOOLBAR),
            wx.NullBitmap, wx.ITEM_NORMAL, u"Save", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_undo = self.m_localToolbar.AddTool(self._undoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_UNDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Undo", wx.EmptyString, None)
        self.m_redo = self.m_localToolbar.AddTool(self._redoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_REDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Redo", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_copy = self.m_localToolbar.AddTool(self._copyCode, u"copy",
            wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Copy", wx.EmptyString, None)
        self.m_cut = self.m_localToolbar.AddTool(self._cutCode, u"cut", wx.ArtProvider.GetBitmap(
            wx.ART_CUT, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL, u"Cut", wx.EmptyString, None)
        self.m_paste = self.m_localToolbar.AddTool(self._pasteCode, u"paste",
            wx.ArtProvider.GetBitmap(wx.ART_PASTE, wx.ART_TOOLBAR), wx.NullBitmap,
            wx.ITEM_NORMAL, u"Paste", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_delete = self.m_localToolbar.AddTool(self._deleteCode, u"delete",
            wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Delete", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_fullScreen = self.m_localToolbar.AddTool(self._toggleFull, u"tool",
            wx.ArtProvider.GetBitmap(u"gtk-fullscreen", wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Toggle fullscreen", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_navigator = self.m_localToolbar.AddTool(self._navigator, u"tool", wx.Bitmap(u"app/res/method.xpm",
            wx.BITMAP_TYPE_ANY), wx.NullBitmap, wx.ITEM_NORMAL, u"insert call", wx.EmptyString, None)
        self.m_localToolbar.Realize()
        self.m_toolbarSizer.Add(self.m_localToolbar, 0, wx.ALL, 5)
        self.Layout()
        # Connect Events
        self.m_editor.Bind(wx.EVT_KILL_FOCUS, self.OnLeaveFocus)
        self.m_editor.Bind(wx.EVT_SET_FOCUS, self.OnGetFocus)

    def __init__(self, parent, mainframe, method):
        """Intialization of method editor"""
        super(MethodPane, self).__init__(parent)
        self._mainframe = mainframe
        self._object = method
        #self._types = dict([(x._name, x) for x in method.types])
        self._notebook = parent

        container = method.inner_member_container
        container_cls = container.inner_class
        if container_cls:
            s = [container_cls.scoped]
            scoped = lambda x: (hasattr(x, 'scoped_str') and x.scoped_str(s)
            ) or (hasattr(x, 'scoped') and x.scoped) or x.name
            self._types = dict([scoped(x), x] for x in container.types)
        else:
            self._types = dict([scoped(x), x] for x in container.types)
            scoped = lambda x: (hasattr(x, 'scoped') and x.scoped) or x.name
            self._types = dict([scoped(x), x] for x in container.types)

        if '@' in self._types:
            del self._types['@']
        self.m_editor.SetText(method._content)
        self.m_editor.Initialize(self._types)
        """
        if method._note:
            self.m_editor.AnnotationSetVisible(wx.stc.STC_ANNOTATION_BOXED)
            self.m_editor.AnnotationSetText(0, method.note)"""
        self.m_editor.SetReadOnly(method._readOnly)
        #self.m_editor.SendMsg(2540, wp="Prueba de anotacion", lp=long(0))
        #self.m_editor.SendMsg(2548, wp=1, lp=long(0))
        self.Layout()
        self._create_toolbars()

        wx.stc.EVT_STC_CHANGE(self.m_editor,
            self.m_editor.GetId(), self.OnEditorChange)

        #self.Bind(EVT_STC_STYLENEEDED, self.handleStyleNeeded,
        #id=self.m_editor.GetId())
        frame = context.frame

        #bind some messages to this frame
        self.Bind(wx.EVT_MENU, frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        self.Bind(wx.EVT_MENU, self.OnDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_MENU, self.OnUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_MENU, self.OnRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_MENU, self.OnCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_MENU, self.OnPasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_MENU, self.OnCutCode, id=self._cutCode)
        self.Bind(wx.EVT_MENU, self.OnSaveCode, id=self._saveCode)
        self.Bind(wx.EVT_MENU, self.OnFullScreen, id=self._toggleFull)
        self.Bind(wx.EVT_MENU, self.OnNavigator, id=self._navigator)

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdatePasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCutCode, id=self._cutCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateSaveCode, id=self._saveCode)

        self.m_editor.popup_handler = self
        self._fullScreen = None

        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), ui.ID_TOGGLE_FULLSCREEN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('S'), self._saveCode),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, self._deleteCode)
        ])
        self.SetAcceleratorTable(aTable)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(self._object._content)
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_editor.Destroy()
        self.m_localToolbar.Destroy()

    def Popup(self, editor, text=None):
        """Shows a custom popup"""
        pos = editor.GetCurrentPos()
        code = editor.GetText()[:pos]
        if text:
            # in the future, we need to show a complete floating dialog here
            # we receive "this->" or "object." here ..
            _vars = gather_vars(code, self._types)
            if not _vars:
                return
            if text not in _vars:
                return
            var = _vars[text]
            _type = var['type']
            if type(_type) is cc.Class:
                pos = editor.GetCurrentPos()
                p = editor.PointFromPosition(pos)
                p = editor.ClientToScreen(p)
                dlg = NavigatorDialog(editor, _type, p)
                if dlg.ShowModal() == wx.ID_OK:
                    editor.AddText(dlg.value)
            return
        else:
            menu = wx.Menu()
            menu.Append(self._mainframe.editProperties.GetId(),
                "test menu", "Edit relation properties")
            pos = editor.GetCurrentPos()
            p = editor.PointFromPosition(pos)
            editor.PopupMenu(menu, p)
            menu.Destroy()

    def OnNavigator(self, event):
        """Navigate throug the class structure"""
        pos = self.m_editor.GetCurrentPos()
        p = self.m_editor.PointFromPosition(pos)
        p = self.m_editor.ClientToScreen(p)
        dialog = app.ui.dlg.CodeNavigator(self, self._object)
        dialog.Move(p)
        if dialog.ShowModal() != wx.ID_OK:
            return
        pass

    def OnUndoCode(self, event):
        """Edit undo"""
        self.m_editor.Undo()

    def OnRedoCode(self, event):
        """Edit redo"""
        self.m_editor.Redo()

    def OnUpdatePasteCode(self, event):
        """Edit paste?"""
        event.Enable(self.m_editor.CanPaste())

    def OnUpdateCopyCode(self, event):
        """Edit copy?"""
        (b, e) = self.m_editor.GetSelection()
        event.Enable(b != e)

    def OnUpdateCutCode(self, event):
        """Edit cut?"""
        if self._object._readOnly:
            event.Enable(False)
        else:
            (b, e) = self.m_editor.GetSelection()
            event.Enable(b != e)

    def OnUpdateRedoCode(self, event):
        """Edit redo?"""
        if self.m_editor.CanRedo():
            event.SetText("Redo edit")
            event.Enable(True)
        else:
            event.SetText("Can't redo")
            event.Enable(False)

    def OnUpdateUndoCode(self, event):
        """Edit undo?"""
        if self.m_editor.CanUndo():
            event.SetText("Undo edit")
            event.Enable(True)
        else:
            event.SetText("Can't undo")
            event.Enable(False)

    def OnUpdateDeleteCode(self, event):
        """Edit delete?"""
        event.Enable(not self._object._readOnly)

    def Enabled(self, event):
        """Joker event"""
        event.Enable(True)

    def Disabled(self, event):
        """Joker event"""
        event.Enable(False)

    def UpdateModifiedTitle(self):
        """Update the tile after modifications"""
        if not self._fullScreen:
            i = self._notebook.GetPageIndex(self)
            s = self._notebook.GetPageText(i)
            m = '[modified] '
            if self.m_editor.IsModified():
                if s.find(m) == wx.NOT_FOUND:
                    self._notebook.SetPageText(i, m + s)
            else:
                if s.find(m) != wx.NOT_FOUND:
                    s = s.replace(m, '')
                    self._notebook.SetPageText(i, s)

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if not self._fullScreen:
            self.UpdateModifiedTitle()

    @tran.TransactionalMethod('edit method')
    def Commit(self):
        """Update document"""
        text = self.m_editor.GetText()
        if text == self._object._content:
            return False
        o = self._object
        o.SaveState()
        o._content = text
        o.project.SetModified(True)
        k = o.outer_class or o.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

        return True

    def OnDeleteCode(self, event):
        """Handle delete key"""
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnCopyCode(self, event):
        """Handle a copy event"""
        self.m_editor.Copy()

    def OnPasteCode(self, event):
        """Handle a paste event"""
        self.m_editor.Paste()

    def OnCutCode(self, event):
        """Handle a paste event"""
        self.m_editor.Cut()

    def OnUpdateSaveCode(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.IsModified())

    def OnSaveCode(self, event):
        """Handles save event"""
        self.Commit()
        self.m_editor.ResetModified()
        self.UpdateModifiedTitle()

    def OnFullScreen(self, event):
        """ShowFullscreen"""
        if self._fullScreen:
            self._fullScreen.LeaveFullScreen()
            self._fullScreen.Destroy()
            self._fullScreen = None
            self.UpdateModifiedTitle()
        else:
            self._fullScreen = pane.FullScreen(self)
        self.m_editor.SetFocus()

    def handleStyleNeeded(self, event):
        """Handle scintylla events"""
        wnd = self.m_editor
        pos = wnd.GetCurrentPos()
        if pos == 0:
            return
        if wnd.GetCharAt(pos - 1) == wx.T(':'):
            lnno = wnd.GetCurrentLine()
            iden = wnd.GetLineIndentation(lnno - 1)
            wnd.SetLineIndentation(lnno, iden)

            # = self.m_editor.GetFoldLevel(lnno - 1)
        # if this is not a normal line
        #self.m_editor.SetFoldLevel(lnno,LastIndent)
        #pass





# -*- coding: utf-8 -*-

import wx
import wx.py.shell


class pythonTextEntry(wx.py.shell.Shell):
    """wxTextCtrl override for execute python code"""
    def __init__(self, parent, id=wx.ID_ANY, title=wx.EmptyString,
        pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TE_MULTILINE |
            wx.TE_PROCESS_ENTER, validator=wx.DefaultValidator, name=wx.TextCtrlNameStr):
        """Initialize the control"""
        glb = globals()
        _l = {'__builtins__': glb['__builtins__'], '__name__': '__main__', '__doc__': None, '__package__': None}
        super(pythonTextEntry, self).__init__(parent, id, locals=_l)













# -*- coding: utf-8 -*-
from model.writer import writer


class memwriter(writer):
    """memwriter is a comodity class for writing C/C++ files"""
    def __init__(self, language="c++"):
        """initialization"""
        super(memwriter, self).__init__(language)
        self.data = []

    def writeln(self, txt=''):
        """write a line or secuence of lines with ident"""
        for l in txt.splitlines():
            self.data.append("{0}{1}\n".format(self.cident, l))

    def writenl(self, count=1):
        """write a newline"""
        for i in range(0, count):
            self.data.append('\n')

    def __str__(self):
        """dumps the string"""
        return ''.join(self.data)






# -*- coding: utf-8 -*-
import re
import os
import wx
import model
import tran
import ostools


class File(model.TCommon, tran.TransactionFSObject):
    """Declares a file with transactional support"""
    def __init__(self, **kwargs):
        """init"""
        _file = kwargs['file']
        if not os.path.exists(_file):
            with open(_file, 'a'):
                os.utime(_file, None)
        if 'name' not in kwargs:
            kwargs['name'] = os.path.split(_file)[1]
        kwargs['uri'] = kwargs['parent'].project
        super(File, self).__init__(**kwargs)
        self._generator = None
        self.refresh_generator()
        if self._generator:
            self._readOnly = type(self._generator) not in [model.py.Module]

    def __getstate__(self):
        """Dispatch pickle context"""
        state = super(File, self).__getstate__()
        state['_version'] = 0
        return state

    @property
    def breakpoints(self):
        """return the breakpoints attached to the current file"""
        return self.project.breakpoints(self)

    @property
    def bookmarks(self):
        """return the bookmarks attached to the current file"""
        return self.project.bookmarks(self)

    def Refresh(self):
        """refresh file"""
        if not hasattr(self, '_generator'):
            self._generator = None
        self.refresh_generator()
        self._readOnly = bool(self._generator and
            type(self._generator) not in [model.py.Module])

    def GetText(self):
        """Returns file contents"""
        with open(self.abs_file, "r") as f:
            data = f.read()
        return data

    def SetText(self, text):
        """Change text contents"""
        with open(self.abs_file, "w") as f:
            f.truncate(0)
            f.write(text.encode('utf-8'))
        if not self._generator:
            return
        # if the generator is python, it may be a module or package
        if type(self._generator) is model.py.Module:
            self._generator.SaveState()
            self._generator._content = text.encode('utf-8')
            self._generator.analize()

    def refresh_generator(self):
        """Any file may correspond to any generator that resides in a model.
        This method finds the responsible generator and do alink with it"""
        project = self.project
        r = os.path.realpath
        j = os.path.join
        if project._language == 'c++':
            if re.match(r'(.)*\.cpp', self._file):
                if self._generator and self._generator.abs_path_source == self.abs_file:
                    return
                try:
                    cpp_set = project.cpp_set
                except:
                    s = project.sources_dir
                    plc = project.level_classes
                    cpp_set = dict([(r(j(s, '{}.cpp'.format(c._name))), c) for c in plc])
                generator = cpp_set.get(self.abs_file, None)
                if generator is not self._generator:
                    if self._generator:
                        self._generator.set_source(None)
                    self._generator = generator
                    if self._generator:
                        self._generator.set_source(self)
            elif re.match(r'(.)*\.h', self._file):
                if self._generator and self._generator.abs_path_header == self.abs_file:
                    return
                try:
                    h_set = project.h_set
                except:
                    s = project.headers_dir
                    plc = project.level_classes
                    h_set = dict([(r(j(s, '{}.h'.format(c._name))), c) for c in plc])
                generator = h_set.get(self.abs_file, None)
                if generator is not self._generator:
                    if self._generator:
                        self._generator.set_header(None)
                    self._generator = generator
                    if self._generator:
                        self._generator.set_header(self)
        elif project._language == 'python':
            if re.match(r'(.)*\.py', self._file):
                if self._generator and self._generator.abs_path == self.abs_file:
                    return
                try:
                    file_dict = project.file_dict
                except:
                    file_dict = dict([(r(j(x.dir, '{x._name}.py'.format(x=x))), x)
                        for x in project.modules])
                    file_dict.update([(r(j(x.dir, '__init__.py')), x)
                        for x in project.packages])
                generator = file_dict.get(self.abs_file, None)
                if generator is not self._generator:
                    if self._generator:
                        self._generator.set_file(None)
                    self._generator = generator
                    if self._generator:
                        self._generator.set_file(self)

    @property
    def bitmap_index(self):
        """Returns tree image id"""
        try:
            return ostools.MimeHandler.file_image_index(self.abs_file)
        except:
            return wx.NOT_FOUND

    @property
    def is_binary(self):
        """Check if this is a binary file"""
        textchars = bytearray({7, 8, 9, 10, 12, 13, 27} | set(range(0x20, 0x100)) - {0x7f})
        is_binary_string = lambda bytes: bool(bytes.translate(None, textchars))
        with open(self.abs_file, "r") as f:
            for line in f:
                if is_binary_string(line):
                    return True
        return False

    def Delete(self):
        """Delete """
        #new version
        if self._generator:
            if type(self._generator) is model.cc.Class:
                if hasattr(self._generator, '_header_obj') and self._generator._header_obj == self:
                    self._generator.set_header(None)
                if hasattr(self._generator, '_source_obj') and self._generator._source_obj == self:
                    self._generator.set_source(None)
        super(File, self).Delete()



# -*- coding: utf-8 -*-

from tools import edit
from tools import append_menuitem_copy
from tools import set_menu_handlers
from tools import unset_menu_handlers
from tools import clone_mnu

from ctrl import Editor



"""Subclass of NewConstructor, which is generated by wxFormBuilder."""

import wx
import wxx
import app.resources as rc
from activity.models.ui import ui as ui

# Implementing NewConstructor
class ConstructorDialog(ui.NewConstructor):
    """
    This dialog allows to create a new class constructor and
    his attributes. The preferred constructor means that this
    constructor will be chosen by default for the list of
    the required arguments of derived classes and creating the
    init sections of these classes.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(ConstructorDialog, self).__init__(parent)
        self._container = container
        self._class = container.inner_class
        self.m_choice2.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("constructor"))
        self.SetIcon(icon)

    def CopyAttributes(self, ctor):
        """Transfer attributes to ctor"""
        ctor._name = self._name
        ctor._access = self._access
        ctor._explicit = self._explicit
        ctor._inline = self._inline
        ctor._declare = self._declare
        ctor._implement = self._implement
        ctor._calling = self._calling
        if self._calling:
            ctor._callingText = self._callingText
        ctor.note = self._note
        ctor.SetPreferred(self._preferred)

    def SetAttributes(self, ctor):
        """Transfer attributes from existing constructor"""
        self._container = ctor.parent
        self._class = ctor.inner_class
        self._name = ctor._name
        self._access = ctor._access
        iSel = self.m_choice2.FindString(self._access)
        self.m_choice2.Select(iSel)
        self._declare = ctor._declare
        self.m_checkBox92.SetValue(self._declare)
        self._implement = ctor._implement
        self.m_checkBox93.SetValue(self._implement)
        self._inline = ctor._inline
        self.m_checkBox31.SetValue(self._inline)
        self._calling = ctor._calling
        self.m_checkBox6.SetValue(self._calling)
        self.m_comboBox1.Enable(self._calling)
        if self._calling:
            self._callingText = ctor._callingText
            iSel = self.m_comboBox1.FindString(self._callingText)
            self.m_comboBox1.Select(iSel)
        self._note = ctor.note
        self.m_richText1.SetValue(self._note)
        self._preferred = ctor.IsPreferred()
        pref = self._class.GetPreferredConstructor()
        if pref is None or pref is ctor:
            self.m_checkBoxPreferred.Enable(True)
        self.m_checkBoxPreferred.SetValue(self._preferred)
        self.SetTitle("Edit constructor")

    def Validate(self):
        """Validation"""
        self._name = self._class._name
        iSel = self.m_choice2.GetSelection()
        self._access = self.m_choice2.GetString(iSel)
        self._declare = self.m_checkBox92.IsChecked()
        self._explicit = self.m_checkBox41.IsChecked()
        self._inline = self.m_checkBox31.IsChecked()
        self._implement = self.m_checkBox93.IsChecked()
        self._calling = self.m_checkBox6.IsChecked()
        if self._calling:
            iSel = self.m_comboBox1.GetSelection()
            self._callspec = self.m_comboBox1.GetString(iSel)
        else:
            self._callspec = ''
        self._note = self.m_richText1.GetValue()
        self._preferred = self.m_checkBoxPreferred.IsChecked()
        return True

    def get_kwargs(self):
        """return arguments suitable for creation"""
        return {'parent': self._container, 'name': self._name,
            'access': self._access, 'explicit': self._explicit,
            'inline': self._inline, 'declare': self._declare,
            'implement': self._implement, 'calling': self._calling,
            'callspec': self._callspec, 'note': self._note,
            'preferred': self._preferred, 'autoargs': True}

    def OnCallingToggle(self, event):
        """Implements calling toggle"""
        self.m_comboBox1.Enable(self.m_checkBox6.IsChecked())

    # Handlers for NewConstructor events.
    def OnCancel(self, event):
        """OnCancel"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





"""Subclass of FindInFiles, which is generated by wxFormBuilder."""

import wx
import wxx
import app


# Implementing FindInFiles
class FindInFilesDialog(app.FindInFiles):
    """
    This dialog allows to find for a text
    inside files. The results will be
    show in the search pane.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container=None):
        """Initialize dialog"""
        self.container = container
        super(FindInFilesDialog, self).__init__(parent)

    def Validate(self):
        """Validate the dialog"""
        self.searchString = self.m_search_string.GetValue()
        return bool(self.searchString)

    def OnOK(self, event):
        """Handler for Ok button press"""
        if self.Validate():
            self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        """Handler for Cancel button press"""
        self.EndModal(wx.ID_CANCEL)




# -*- coding: utf-8 -*-
from GitRepo import GitRepoDialog
from OpenGitRepo import OpenGitRepoDialog
from CommitGit import CommitGitDialog
from GitRemote import GitRemoteDialog
from Progress import ProgressDialog


# -*- coding: utf-8 -*-

"""Git remote representation"""

import model
import app.resources as rc


class GitRemote(model.TComponent):
    """Implements git remote representation"""

    def __init__(self, **kwargs):
        """Initialization method"""
        self._url = kwargs.get('url', None)
        self._pass = kwargs.get('pass', None)
        super(GitRemote, self).__init__(**kwargs)
        if kwargs.get('new', False):
            try:
                # The remote repo must be added
                self.repo._repo.create_remote(self._name, self._url)
            except:
                pass

    def update_status(self):
        """update the remote status"""
        pass

    @property
    def repo(self):
        """get the local repo object"""
        return self.parent.repo

    @property
    def password(self):
        """return password"""
        return self._pass

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {'url': self._url, 'pass': self._pass}
        kwargs.update(super(GitRemote, self).get_kwargs())
        return kwargs

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(GitRemote, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        _repo = self.repo._repo
        try:
            ref = _repo.remote(self.name)
            _repo.delete_remote(ref)
        except:
            pass
        super(GitRemote, self).Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(GitRemote, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(GitRemote, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(GitRemote, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        _repo = self.repo._repo
        try:
            ref = _repo.remote(self.name)
            _repo.delete_remote(ref)
        except:
            pass
        super(GitRemote, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        super(GitRemote, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        try:
            self.repo._repo.create_remote(self._name, self._url)
        except:
            pass
        super(GitRemote, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('git_remote')

    @property
    def label(self):
        """Get tree label"""
        return '{self._name}'.format(self=self)






"""Subclass of Working, which is generated by wxFormBuilder."""

import app
import sys
import os


# Implementing Working
class WorkingDialog(app.Working):
    """Dialog for showing wait"""
    def __init__(self, parent):
        """Initialize dialog"""
        super(WorkingDialog, self).__init__(parent)
        path = os.path.split(sys.argv[0])[0]
        self.m_animCtrl.LoadFile(os.path.join(path, "app/res/wait.gif"))
        self.m_animCtrl.Play()




"""Subclass of AstExplorerPane, which is generated by wxFormBuilder."""

import ast
import wx
import Tool
import activity.files.ui.pane as pane
import res
from ctx import theContext as context
import wx.propgrid as pg


# Implementing AstExplorerPane
class ToolAstExplorerPane(Tool.AstExplorerPane):
    """Ast Explorer pane"""
    def __init__(self, parent, item, data):
        """Init"""
        self._path = item._file
        self._treeid = None
        self._data = data
        Tool.AstExplorerPane.__init__(self, parent)
        self._item = item
        self.m_editor = pane.FilePythonPane(self.m_splitter3, context.frame, self._item)
        self.m_editor.SetReadOnly(True)
        self.m_splitter3.SplitVertically(self.m_panel32, self.m_editor, 0)
        self.m_tree.AssignImageList(res.GetBitmapImageList())
        self.MapIcons()
        self.FillPath()

    def MapIcons(self):
        """Create a image map"""
        self._iconmap = {}
        self._iconmap[ast.Module] = res.GetBitmapIndex('module')
        self._iconmap[ast.Expr] = res.GetBitmapIndex('expr')
        self._iconmap[ast.Import] = res.GetBitmapIndex('import')
        self._iconmap[ast.ImportFrom] = res.GetBitmapIndex('import')
        self._iconmap[ast.Str] = res.GetBitmapIndex('str')
        self._iconmap[ast.alias] = res.GetBitmapIndex('alias')
        self._iconmap[ast.Assign] = res.GetBitmapIndex('assign')
        self._iconmap[ast.Name] = res.GetBitmapIndex('name')
        self._iconmap[ast.BinOp] = res.GetBitmapIndex("binop")
        self._iconmap[ast.Store] = res.GetBitmapIndex("store")
        self._iconmap[ast.Load] = res.GetBitmapIndex("load")
        self._iconmap[ast.Attribute] = res.GetBitmapIndex("attribute")
        self._iconmap[ast.Add] = res.GetBitmapIndex("add")
        self._iconmap[ast.Num] = res.GetBitmapIndex("num")
        self._iconmap[ast.ClassDef] = res.GetBitmapIndex("classdef")
        self._iconmap[ast.FunctionDef] = res.GetBitmapIndex("functiondef")
        self._iconmap[ast.arguments] = res.GetBitmapIndex("arguments")
        self._iconmap[ast.Param] = res.GetBitmapIndex("param")
        self._iconmap[ast.Call] = res.GetBitmapIndex("call")
        self._iconmap[ast.Dict] = res.GetBitmapIndex("dict")
        self._iconmap[ast.Return] = res.GetBitmapIndex("return")
        self._iconmap[ast.ListComp] = res.GetBitmapIndex("listcomp")
        self._iconmap[ast.Tuple] = res.GetBitmapIndex("tuple")
        self._iconmap[ast.comprehension] = res.GetBitmapIndex("comprehension")
        self._iconmap[ast.Global] = res.GetBitmapIndex("global")
        self._iconmap[ast.For] = res.GetBitmapIndex("for")
        self._iconmap[ast.keyword] = res.GetBitmapIndex("keyword")
        self._iconmap[ast.If] = res.GetBitmapIndex("if")
        self._iconmap[ast.BoolOp] = res.GetBitmapIndex("bool")
        self._iconmap[ast.And] = res.GetBitmapIndex("and")
        self._iconmap[ast.Compare] = res.GetBitmapIndex("compare")
        self._iconmap[ast.In] = res.GetBitmapIndex("in")
        self._iconmap[ast.Subscript] = res.GetBitmapIndex("subscript")
        self._iconmap[ast.Index] = res.GetBitmapIndex("index")
        self._iconmap[ast.UnaryOp] = res.GetBitmapIndex("unary")
        self._iconmap[ast.Not] = res.GetBitmapIndex("not")
        self._iconmap[ast.Eq] = res.GetBitmapIndex("eq")
        self._iconmap[ast.NotEq] = res.GetBitmapIndex("not_eq")
        self._iconmap[ast.Delete] = res.GetBitmapIndex("delete")
        self._iconmap[ast.Del] = res.GetBitmapIndex("del")
        self._iconmap[ast.Sub] = res.GetBitmapIndex("sub")
        self._iconmap[ast.NotIn] = res.GetBitmapIndex("not_in")
        self._iconmap[ast.GtE] = res.GetBitmapIndex("gte")
        self._iconmap[ast.LtE] = res.GetBitmapIndex("lte")
        self._iconmap[ast.Gt] = res.GetBitmapIndex("gt")
        self._iconmap[ast.Lt] = res.GetBitmapIndex("lt")
        self._iconmap[ast.Is] = res.GetBitmapIndex("is")
        self._iconmap[ast.IsNot] = res.GetBitmapIndex("is_not")
        self._iconmap[ast.FloorDiv] = res.GetBitmapIndex("floor_div")
        self._iconmap[ast.Continue] = res.GetBitmapIndex("continue")
        self._iconmap[ast.TryExcept] = res.GetBitmapIndex("try")
        self._iconmap[ast.Or] = res.GetBitmapIndex("or")
        self._iconmap[ast.ExceptHandler] = res.GetBitmapIndex("except")
        self._iconmap[ast.Print] = res.GetBitmapIndex("print")
        self._iconmap[ast.List] = res.GetBitmapIndex("list")

    def process_node(self, parent_item, node):
        """This is the root of process analysing"""
        data = wx.TreeItemData(node)
        tree_item = self.m_tree.AppendItem(parent_item, str(type(node)), data=data)
        if type(node) in self._iconmap:
            z = self._iconmap[type(node)]
            self.m_tree.SetItemImage(tree_item, z, wx.TreeItemIcon_Normal)
        #else:
        #    print type(node)
        for child_node in ast.iter_child_nodes(node):
            self.process_node(tree_item, child_node)

    def FillPath(self):
        """Fill the tree with the ast info"""
        self.m_tree.AddRoot('nodes')
        self.process_node(self.m_tree.GetRootItem(), self._data)

    def OnSelectItem(self, event):
        """Process select item"""
        item = self.m_tree.GetSelection()
        if not item.IsOk():
            return
        data = self.m_tree.GetItemData(item)
        obj = data.GetData()
        if hasattr(obj, 'to_lineno'):
            #self.m_editor.GotoLine(obj.lineno - 1, True)
            self.m_editor.Select(obj.lineno - 1, obj.col_offset, obj.to_lineno - 1, obj.to_col_offset)
        self.m_propertyGrid.Clear()
        for k in obj.__dict__.keys():
            prop = pg.NewStringProperty(k, k, str(obj.__dict__[k]))
            self.m_propertyGrid.Append(prop)
            self.m_propertyGrid.SetPropertyReadOnly(k)




"""Subclass of NewEnum, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewEnum
class EnumDialog(ui.NewEnum):
    """
    This dialog allows the edition of enum type.
    If this enum is inside class or struct, you can also
    specify his visibility.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(EnumDialog, self).__init__(parent)
        self._container = container
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("enum"))
        sz = self.m_listCtrl5.GetClientSize()
        w = sz.x // 2 - 4
        self.m_listCtrl5.InsertColumn(0, "items", wx.LIST_FORMAT_LEFT, w)
        self.m_listCtrl5.InsertColumn(1, "values", wx.LIST_FORMAT_LEFT, w)
        if container.inner_class:
            self.m_staticText8.Enable(True)
            self.m_choice2.Enable(True)
        self.SetIcon(icon)

    def SetAttributes(self, obj):
        """Transfer attributes to dialog"""
        self.SetTitle("Edit enum")
        # reset button states
        self.m_bpButton14.Enable(False)
        self.m_bpButton12.Enable(False)
        self.m_bpButton13.Enable(False)
        #remove old elements if any
        self.m_listCtrl5.DeleteAllItems()
        #set name
        self.m_textCtrl55.SetValue(obj._name)
        #set access
        iSel = self.m_choice2.FindString(obj._access)
        self.m_choice2.SetSelection(iSel)
        #insert new elements
        for pair in obj._items:
            self.m_listCtrl5.Append(pair)
        #set note
        self.m_richText3.SetValue(obj.note)
        self.SetTitle("Edit enum")

    def get_kwargs(self):
        """return arguments suitable for object instance"""
        return {'parent': self._container, 'name': self._name,
                'access': self._access, 'items': self._items,
                'note': self._note}

    def CopyAttributes(self, enum):
        """Copy attributes to method"""
        enum._name = self._name
        enum._access = self._access
        enum._items = self._items
        enum.note = self._note

    def Validate(self):
        """Check about validity"""
        self._name = self.m_textCtrl55.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("enum name must be not empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("enum name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        #get the list of values
        self._items = [
            (
                self.m_listCtrl5.GetItemText(i, 0),
                self.m_listCtrl5.GetItemText(i, 1)
            ) for i in range(0, self.m_listCtrl5.GetItemCount())]
        iSel = self.m_choice2.GetCurrentSelection()
        self._access = self.m_choice2.GetString(iSel)
        self._note = self.m_richText3.GetValue()
        return True

    # Handlers for NewEnum events.
    def OnAddEnumItem(self, event):
        """Implement OnAddEnumItem"""
        from EnumItem import EnumItemDialog
        d = EnumItemDialog(self)
        if d.ShowModal() == wx.ID_OK:
            self.m_listCtrl5.Append((d._label, d._value))

    def OnSelectEnumItem(self, event):
        """Handles enum item selection"""
        # When an enum item is selected, we enable delete it
        self.m_bpButton14.Enable(True)
        # if this item is not the first, enable move up
        self.m_bpButton12.Enable(event.GetIndex() > 0)
        # if this item is not the last, enable move down
        self.m_bpButton13.Enable(event.GetIndex() + 1 < self.m_listCtrl5.GetItemCount())

    def OnDeselectEnumItem(self):
        """Handle enum item unselect"""
        self.m_bpButton14.Enable(False)
        self.m_bpButton12.Enable(False)
        self.m_bpButton13.Enable(False)

    def OnEditEnumItem(self, event):
        """Handles enum item edition"""
        i = event.GetIndex()
        label = self.m_listCtrl5.GetItemText(i, 0)
        value = self.m_listCtrl5.GetItemText(i, 1)
        from EnumItem import EnumItemDialog
        d = EnumItemDialog(self, (label, value))
        if d.ShowModal() == wx.ID_OK:
            item = wx.ListItem()
            item.SetId(i)
            item.SetMask(wx.LIST_MASK_TEXT)
            item.SetText(d._label)
            item.SetColumn(0)
            self.m_listCtrl5.SetItem(item)
            item.SetMask(wx.LIST_MASK_TEXT)
            item.SetText(d._value)
            item.SetColumn(1)
            self.m_listCtrl5.SetItem(item)

    def OnEnumItemUp(self, event):
        """Implement OnEnumItemUp"""
        i = self.m_listCtrl5.GetFirstSelected()
        label = self.m_listCtrl5.GetItemText(i, 0)
        value = self.m_listCtrl5.GetItemText(i, 1)
        self.m_listCtrl5.DeleteItem(i)
        i = i - 1
        self.m_listCtrl5.InsertStringItem(i, label)
        self.m_listCtrl5.SetStringItem(i, 1, value)
        self.m_listCtrl5.Select(i)

    def OnEnumItemDown(self, event):
        """Implement OnEnumItemDown"""
        i = self.m_listCtrl5.GetFirstSelected()
        label = self.m_listCtrl5.GetItemText(i, 0)
        value = self.m_listCtrl5.GetItemText(i, 1)
        self.m_listCtrl5.DeleteItem(i)
        i = i + 1
        self.m_listCtrl5.InsertStringItem(i, label)
        self.m_listCtrl5.SetStringItem(i, 1, value)
        self.m_listCtrl5.Select(i)

    def OnRemoveEnumItem(self, event):
        """Implement OnRemoveEnumItem"""
        i = self.m_listCtrl5.GetFirstSelected()
        self.m_listCtrl5.DeleteItem(i)

    def OnCancel(self, event):
        """Cancel dialog"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Implement OnOK"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-

import os
import xml.etree.ElementTree as et

class Test:

    def __init__(self):
        self._name = 'Ozores'

v = Test()
dictionary={}
dictionary['toRel']=v

z=eval('toRel',dictionary)
print(z)
print(v)
exit()

tree = et.parse(
    '/home/mel/pythonProjects/proCxx/plugin/models/relation/standard/toMethods.xml')
root = tree.getroot()
definitions = root.find('definitions')
if definitions is not None:
    for definition in definitions:
        text = definition.findtext('.')
        print tuple(tuple(text.split('=')))
        for x, y in [text.split('=')]:
            v = y.format(**dictionary)
            dictionary[x] = v
        print dictionary



"""Subclass of NewClass, which is generated by wxFormBuilder."""

import re
import wx
import wxx
import model
import app.resources as rc
from activity.models.ui import ui as ui


# Implementing NewClass
class ClassDialog(ui.NewClass):
    """
    This dialog allows to create a new class or struct. Even as top
    element or nested in anoter project container. You can specify
    access for child classes and structs. Serialize flag indicates
    the creation and mainteinance of serialization methods for the
    class.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialice dialog"""
        super(ClassDialog, self).__init__(parent)
        self.m_textCtrl2.SetFocus()
        self._container = container
        self.project = container.project
        self._original = ""
        self._template = None
        self._template_types = []
        self._is_external = False
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("class"))
        self.SetIcon(icon)
        self.m_textCtrl2.SetFocus()
        if container.inner_class:
            self.m_staticText88.Enable()
            self.m_choice2.Enable()

    def OnToggleTemplate(self, event):
        """Handles toggle event"""
        if self.m_checkBox61.GetValue():
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
        else:
            self.m_stStartTemplate.Enable(False)
            self.m_template.SetValue('')
            self.m_template.Enable(False)
            self.m_stEndTemplate.Enable(False)
        event.Skip()

    def OnChangeTemplateSpecification(self, event):
        """Handles change text event"""
        text = self.m_template.GetValue()
        l = text.split(',')
        prologs = ['class', 'typename']
        r = []
        for u in l:
            v = u.split('=')[0].strip()
            for p in prologs:
                if v.find(p) == 0:
                    v = v.replace(p, '').strip()
                    if len(v):
                        r.append(v.strip())
                    break
        # set templates to available types
        self._template_types = r
        event.Skip()

    def OnToggleExternalClass(self, event):
        """Handle the external flag"""
        self._is_external = self.m_checkBox90.GetValue()
        self.m_checkBox60.Enable(not self._is_external)
        event.Skip()

    def Validate(self):
        """implement OnOK button"""
        self._name = self.m_textCtrl2.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Class name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("class name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if not self._original == self._name:
            if self._name in [x._name for x in self.project.level_classes]:
                wx.MessageBox("Class already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._is_struct = self.m_checkBox83.GetValue()
        self._note = self.m_richText3.GetValue()
        self._memberPrefix = self.m_textPrefix.GetValue()
        if self.m_checkBox61.GetValue():
            self._template = self.m_template.GetValue()
        else:
            self._template = None
        self._access = self.m_choice2.GetStringSelection()
        return True

    def CopyAttributes(self, cls):
        """Copy dialog selections to object"""
        cls._is_external = self._is_external
        cls._name = self._name
        cls._is_struct = self._is_struct
        cls.note = self._note
        cls._memberPrefix = self._memberPrefix
        cls._template = self._template
        cls._template_types = self._template_types
        cls._access = self._access

    def SetAttributes(self, clase):
        """Copy object attributes to dialog"""
        self.SetTitle("Edit class/struct")
        #backward compatibility code - to simplfy in newer versions
        if not hasattr(clase, '_is_external'):
            clase._is_external = False
        self._is_external = clase._is_external
        self._original = clase._name
        self._name = clase._name
        self._note = clase.note
        self._is_struct = clase._is_struct
        self._memberPrefix = clase._memberPrefix
        self._template = clase._template
        self._template_types = clase._template_types
        self.m_textCtrl2.SetValue(self._name)
        self.m_checkBox83.SetValue(self._is_struct)
        self.m_textPrefix.SetValue(self._memberPrefix)
        self.m_richText3.SetValue(self._note)
        self._access = clase._access
        if clase.parent.inner_class:
            self.m_staticText88.Enable()
            self.m_choice2.Enable()
            i = self.m_choice2.FindString(self._access)
            self.m_choice2.Select(i)

        if clase._template is not None:
            self.m_stStartTemplate.Enable(True)
            self.m_template.Enable(True)
            self.m_stEndTemplate.Enable(True)
            self.m_checkBox61.SetValue(True)
            self.m_template.SetValue(self._template)
            # if template is specified and some contained method /argument holds
            # some of the template types, the template specification cannot be changed
            self.SetTitle("Edit class")
            for method in clase(model.cc.MemberMethod):
                if method._typei._type_alias in clase._template_types:
                    self.m_checkBox61.Enable(False)
                    self.m_checkBox61.SetToolTipString(
                        'Template cannot be edited because some method is using this template types.')
                    self.m_stStartTemplate.Enable(False)
                    self.m_template.Enable(False)
                    self.m_stEndTemplate.Enable(False)
                    return
            for arg in clase(model.cc.Argument):
                if arg._typei._type_alias in clase._template_types:
                    self.m_checkBox61.Enable(False)
                    self.m_checkBox61.SetToolTipString(
                        'Template cannot be edited because some argument is using this template types.')
                    self.m_stStartTemplate.Enable(False)
                    self.m_template.Enable(False)
                    self.m_stEndTemplate.Enable(False)
                    return
        self.m_checkBox90.SetValue(self._is_external)

    def get_kwargs(self):
        """Returns suitable arguments for creating object"""
        kwargs = {}
        kwargs['parent'] = self._container
        kwargs['name'] = self._name
        kwargs['is_struct'] = self._is_struct
        kwargs['access'] = self._access
        kwargs['template'] = self._template
        kwargs['template_types'] = self._template_types
        kwargs['prefix'] = self._memberPrefix
        kwargs['note'] = self._note
        kwargs['is_external'] = self._is_external
        return kwargs

    def OnOK(self, event):
        """implement OnOK button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-
import wx


class EditionDialog(object):
    """Class decorator for edition dialog. With this decorator
    the decorated method only needs to send the dialog arguments"""
    def __init__(self, dialogType):
        """"""
        self._dialogType = dialogType

    def __call__(self, method):
        """"""
        import tran.TransactionStack as stack

        def wrapped_call(*args, **kwargs):
            """Code dialog edition"""
            (wnd, obj) = method(*args, **kwargs)
            dialog = self._dialogType(wnd, obj.parent)
            dialog.SetAttributes(obj)
            if dialog.ShowModal() != wx.ID_OK:
                return False
            obj.SaveState()
            dialog.CopyAttributes(obj)
            if obj.project:
                obj.project.SetModified(True)
            if obj and stack.InTransaction() and hasattr(obj, '_name'):
                sname = stack.instance.GetName()
                if sname:
                    sname = sname.format(obj._name)
                    stack.instance.SetName(sname)
            return True
        return wrapped_call


# -*- coding: utf-8 -*-
import wx
import re

class CodeMap(object):
    """Class for analysis of code. The main target
    for this method is to make the analysis of the
    method code previous to any editor point. The
    analyser, for that reason, dont expect correct
    code."""
    def __init__(self, macrodef={}):
        """initialize codemap"""
        self._language = "C++"
        self._version = "03"
        self.no_for_scope = False
        self.strmap = {}
        self.macrodef = macrodef

        #regexpr
        self._forexpr = re.compile(r'\bfor\b\s*\(([^;\)]*);([^;\)]*);([^;\)]*)\)')
        self._whilexpr = re.compile(r'\bwhile\b\s*\([^\)]*\)')

        #setup
        self.inString = False
        self.inLineComment = False
        self.inTextComment = False
        # clean macros
        for macro in self.macrodef:
            self.macrodef[macro] = self.join_splits(self.macrodef[macro])

        super(CodeMap, self).__init__()

    def remove_comments(self, code):
        """Return code without comments"""
        left = ''
        right = code.strip()
        while len(right):
            starts = [right.find(s) for s in ['//', '/*', '"']]
            i = 0
            v = 0
            try:
                v = min([x for x in starts if x > -1])
            except:
                #there are no comments
                return left + right
            i = starts.index(v)
            if i == 0:
                #is a C++ line comment
                left += right[:v]
                right = right[v:]
                ends = [right.find(s) for s in ['\r', '\n']]
                try:
                    v = min([x for x in ends if x > -1])
                except:
                    # is the last line, no continuation
                    self.inLineComment = True
                    return left
                if v + 1 < len(right):
                    if right[v + 1] in '\r\n':
                        v += 1
                right = right[v + 1:]
                continue
            if i == 1:
                #is a C compound line comment
                left += right[:v]
                right = right[v:]
                v = right.find('*/')
                if v == wx.NOT_FOUND:
                    #be patient ...
                    #raise SyntaxError("unclosed comment ...{}".format(left[-10:]))
                    self.inTextComment = True
                    return left
                right = right[v + 2:]
            else:
                #It's a string
                #v = v + 1
                # replace string by @str<num>
                si = '@str{}'.format(len(self.strmap) + 1)
                left += right[:v]
                left += si
                right = right[v + 1:]
                v = right.find('"')
                if v == wx.NOT_FOUND:
                    self.strmap[si] = right
                    #we are analysing incomplete code,
                    #keep that in mind ...
                    #raise SyntaxError("unclosed string ...{}".format(left[-10:]))
                    self.inString = True
                    return left
                self.strmap[si] = right[:v]
                v = v + 1
                #left += right[:v]
                right = right[v:]
        return left + right

    def remove_macro_alternative(self, code):
        """
        Remove inactive by macro. When we are coding something
        and we are inside the #else part of macro condition, we can
        completely discard all the code inside the #if block.
        Also, if we have completed #if/#else/#endif sections inside
        the analysed code, the variables defined with the same name
        in both of the blocks will be in conflict if his types are
        not compatibles, or downgraded to the common base class for
        the declarations.
        So, when analysing macro conditions not only we do the removal
        of #if sections from open #else's, but also check the conflicts
        and base types for matching name vars.
        In resume,
            for complete #if/#else/#endif
                - downgrade of matching name vars to high common base
                - warn unmatching vars
                - warn missing vars in some counterpart
            for incomplete #else
                - check of already declared vars with #if counterpart
                - removal of #if code
                - warn missing vars"""

        left = ''
        right = code
        count = 0
        kpos = {}
        while len(right) > 0:
            # find delimitters
            d = [right.find(x) for x in ['#if', '#endif']]
            try:
                v = min([x for x in d if x > -1])
            except:
                # there are not more conditions
                # check about the else presence
                if count:
                    d = right.find('#else')
                    if d > -1:
                        left += right[:d]
                        right = right[d:]
                        #remove macro
                        d = right.find('\n') + 1
                        if d > 0:
                            right = right[d:]
                        else:
                            right = right[5:]
                        #we can discard the inner #if:

                return left + right
            left += right[:v]
            right = right[v:]
            i = d.index(v)
            if i == 0:
                count += 1
                kpos[count] = len(left)
                left += "#if"
                right = right[3:]
            elif i == 1:
                left += "#endif"
                right = right[6:]
                count -= 1

    def remove_control(self, code):
        """Remove control sentences from text"""
        # 1º for(<init>;<cond>;<iter>){ will be transformed into
        #    <init>;{ or {<init>; depending on no_for_scope
        it = self._forexpr.finditer(code)
        matches = []
        initializers = []
        remove = []
        for match in it:
            span = match.span()
            #if this is a single line statement, ignore it
            k = code[span[1]:].strip()
            remove.append(len(k) == 0 or k[0] != '{')
            matches.append(span)
            initializers.append(match.group(1))
        # Ok, proceed to reemplace code
        s = ''
        last = len(code)
        for i in range(len(matches) - 1, -1, -1):
            span = matches[i]
            if not remove[i]:
                s = code[span[1]:last] + s
                if self.no_for_scope:
                    s = '{};'.format(initializers[i]) + s
                else:
                    #search pos
                    k = s.find('{')
                    s = '{{{};'.format(initializers[i]) + s[k + 1:]
            else:
                # move last to skip line
                k = code.find(';', span[1])
                if k > -1:
                    s = code[k + 1:last] + s
            last = span[0]
        code = code[:last] + s
        # 2º Ok, now we will remove while sentences
        matches = [x.span() for x in self._whilexpr.finditer(code)]
        s = ''
        last = len(code)
        for i in range(len(matches) - 1, -1, -1):
            s = code[matches[i][1]:last] + s
            last = matches[i][0]
        code = code[:last] + s
        return code

    def remove_closed_statements(self, code):
        """Each closed statement dont contribute for context variables"""
        o = code.split('{')
        s = 0
        for i in range(0, len(o)):
            v = o[i + s].split('}')
            for j in range(1, len(v)):
                if i + s < 0:
                    continue
                    #strictly speaking is an error, but we need
                    #to be rude people here ...
                    #raise SyntaxError("unbalanced statements")
                del o[i + s]
                s = s - 1
        #ok, recompose and return
        s = ""
        for i in range(0, len(o)):
            s += o[i] + "{"
        return s[:-1]

    def join_splits(self, code):
        """remove linesplits (ie. end line with '\' character) by
        joining that lines"""
        code = code.replace('\\\r\n', '')
        code = code.replace('\\\r', '')
        code = code.replace('\\\n', '')
        return code

    def apply_macrodef(self, macro, code):
        """Apply any macro to code"""
        if macro in code and macro in self.macrodef:
            code = code.replace(macro, self.macrodef[macro])
            code = self.remove_comments(code)
        return code

    def capture_vars(self, code):
        """Get variables in the code, with type"""
        # The input code only holds active statements
        # so his structure is <code>{<code>{...
        # without closed statement. The process now
        # is easy. First, replace remaining open statements
        # with ;
        code = code.replace('{', ';')
        # now, split in stements and process each one
        statements = code.split(';')
        for s in statements:
            self.process_statement(s)

    def process_statement(self, statement):
        """Process an statement and extract variables"""
        s = statement.strip()
        if len(s) == 0:
            return
        #remove macros
        while s[0] == '#':
            i = s.find('\n')
            if i < 0:
                return
            s = s[i + 1:]
            if not len(s):
                return
        #we dont accept embedded typedef's
        #classes or struct's for now.
        if re.match('\btypedef\b.*'):
            return
        if re.match('\bclass\b.*'):
            return
        if re.match('\bstruct\b.*'):
            return

        # process enums




    def body_analyse(self, code):
        """Analyse body of method"""
        # remove line splits
        code = self.join_splits(code)
        # remove comments and extract strings for simplify
        code = self.remove_comments(code)
        # remove macro counterparts
        code = self.remove_macro_alternative(code)
        for macro in self.macrodef:
            code = self.apply_macrodef(macro, code)
        # remove control sentences
        code = self.remove_control(code)
        # closed statements dont contribute with declarations
        code = self.remove_closed_statements(code)
        self.capture_vars(code)
        return code




# -*- coding: utf-8 -*-


def with_name(function):
    """method decorator that provides self name access"""
    def wrap(self, *args, **kwargs):
        """wrapped calls"""
        self.__fname__ = function.__name__
        return function(self, *args, **kwargs)
    return wrap


def safe_super(_class, _inst):
    """safe super call that resolves for the same method
    even with multiple inheritance"""
    try:
        return getattr(super(_class, _inst), _inst.__fname__)
    except:
        return lambda *x, **kx: ()








"""Subclass of NewBoostPythonModule, which is generated by wxFormBuilder."""

import wx
import re
import wxx
import app.resources as rc
import model.cc
from activity.models.ui import ui as ui


# Implementing NewBoostPythonModule
class BoostPythonModuleDialog(ui.NewBoostPythonModule):
    """
    Using this dialog you can export classes and methods
    from your project using boost.python.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        self._container = container
        self._project = container.project
        self._eert = {}
        super(BoostPythonModuleDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("pyboost"))
        self.SetIcon(icon)
        # dialog initialization: We insert all the classes and methods
        # from the project
        self.m_textCtrl21.SetValue(self._project.name)
        self.imglist = rc.GetBitmapImageList()
        self.m_tree.SetImageList(self.imglist)
        self._root = self.m_tree.AddRoot(self._project.name, ct_type=1, data=None)
        self.m_tree.SetItemImage(self._root, image=self._project.bitmap_index)
        self.append_classes(self._root, self._project)
        self.append_functions(self._root, self._project)

    def append_classes(self, pid, container):
        """Append classes to tree"""
        cls = container.inner_class
        f = lambda x: x.parent.inner_class == cls
        for cls in container(model.cc.Class, filter=f, cut=True):
            if cls._template:
                continue
            item = self.m_tree.AppendItem(pid, cls.tree_label, ct_type=1, data=cls)
            self._eert[cls] = item
            self.m_tree.SetItemImage(item, image=cls.bitmap_index)
            # add nested classes
            # future : self.append_classes(item, cls)
            self.append_methods(item, cls)

    def append_functions(self, pid, container):
        """Append functions to tree"""
        for fn in container(model.cc.Function):
            if fn._template:
                continue
            item = self.m_tree.AppendItem(pid, fn.tree_label, ct_type=1, data=fn)
            self._eert[fn] = item
            self.m_tree.SetItemImage(item, image=fn.bitmap_index)

    def append_methods(self, pid, container):
        """Append functions to tree"""
        cls = container.inner_class
        f = lambda x: x.parent.inner_class == cls
        classes = (model.cc.MemberMethod, model.cc.Constructor, model.cc.IsClassMethod)
        for fn in container(*classes, filter=f, cut=True):
            if fn._template:
                continue
            item = self.m_tree.AppendItem(pid, fn.tree_label, ct_type=1, data=fn)
            self._eert[fn] = item
            self.m_tree.SetItemImage(item, image=fn.bitmap_index)

    def is_checked(self, obj):
        """Check if the corresponding tree element is checked"""
        return self.m_tree.IsItemChecked(self._eert[obj])

    def is_included_class(self, cls):
        """Check if some class must be included or not in exports"""
        if self.is_checked(cls):
            return True
        # but even if the class is not included, if some method is included,
        # the class must be included
        f = lambda x: x.parent.inner_class == cls
        classes = (model.cc.MemberMethod, model.cc.Constructor, model.cc.IsClassMethod)
        try:
            (fn for fn in cls(*classes, filter=f, cut=True) if self.is_checked(fn)).next()
            return True
        except:
            return False

    def scan_classes(self, container, tabs=0):
        """Genera el codigo que corresponde a la exportacion de las clases"""
        cls = container.inner_class
        f = lambda x: x.parent.inner_class == cls
        content = ''
        self._uids = []
        for cls in container(model.cc.Class, filter=f, cut=True):
            if self.is_included_class(cls):
                self._uids.append(cls.uid)
                members = self.scan_member(cls, tabs + 1)
                note = cls.note
                if note:
                    header = '{prefix}class_<{name}>("{name}", "{note}", no_init)'
                else:
                    header = '{prefix}class_<{name}>("{name}", no_init)'
                header = header.format(
                    prefix='\t' * tabs,
                    name=cls.name,
                    note=note
                    )
                content += '{header}\n{members}{prefix};\n'.format(
                    prefix='\t' * tabs,
                    header=header,
                    members=members)
        return content

    def scan_member(self, container, tabs=0):
        """Genera el codigo que corresponde a la exportacion de los miembros de clase"""
        cls = container.inner_class
        f = lambda x: x.parent.inner_class == cls
        classes = (model.cc.MemberMethod, model.cc.Constructor, model.cc.IsClassMethod)
        content = ''
        methods = container(*classes, filter=f, cut=True)
        ctors = [x for x in methods if type(x) is model.cc.Constructor]
        other = [x for x in methods if type(x) is not model.cc.Constructor]
        for fn in ctors:
            if not self.is_checked(fn):
                continue
                # We process a ctor
            self._uids.append(fn.uid)
            args = fn[model.cc.Argument]
            note = fn.note
            if args:
                arglist = ', '.join(['"{name}"'.format(name=x.name) for x in args])
                argTemp = ', '.join([str(x._typei).format('') for x in args])
                if note:
                    expr = '{prefix}.def(init<{argTemp}>(args({arglist}), "{note}")\n'
                else:
                    expr = '{prefix}.def(init<{argTemp}>(args({arglist}))\n'
            else:
                arglist = ''
                argTemp = ''
                if note:
                    expr = '{prefix}.def(init("{note}")\n'
                else:
                    expr = '{prefix}.def(init())\n'
            content += expr.format(
                prefix='\t' * tabs,
                argTemp=argTemp,
                arglist=arglist,
                note=note)
        for fn in other:
            if not self.is_checked(fn):
                continue
            self._uids.append(fn.uid)
            # We process a standard method
            # check arguments
            args = fn[model.cc.Argument]
            note = fn.note
            if args:
                arglist = ', '.join(['arg("{name}")'.format(name=x.name) for x in args])
                if note:
                    expr = '{prefix}.def("{name}", &{clsname}::{name}, ({arglist}), "{note}")\n'
                else:
                    expr = '{prefix}.def("{name}", &{clsname}::{name}, ({arglist}))\n'
            else:
                arglist = ''
                if note:
                    expr = '{prefix}.def("{name}", &{clsname}::{name}, "{note}")\n'
                else:
                    expr = '{prefix}.def("{name}", &{clsname}::{name})\n'
            content += expr.format(
                prefix='\t' * tabs,
                name=fn.name,
                clsname=cls.name,
                arglist=arglist,
                note=note
                )
        return content

    def SetAttributes(self, bpm):
        """Copy object attributes to dialog"""
        self.m_textCtrl21.SetValue(bpm._name)
        for obj in self._eert:
            if type(obj) is model.cc.MemberMethod:
                continue
            if obj.uid in bpm._uids:
                self.m_tree.CheckItem(self._eert[obj], True)
            else:
                self.m_tree.CheckItem(self._eert[obj], False)
        for obj in self._eert:
            if type(obj) is not model.cc.MemberMethod:
                continue
            if obj.uid in bpm._uids:
                self.m_tree.CheckItem(self._eert[obj], True)
            else:
                self.m_tree.CheckItem(self._eert[obj], False)
        self.SetTitle("Edit boost.python module")

    def CopyAttributes(self, bpm):
        """Copy dialog selections to object"""
        content = self.scan_classes(self._project, 1)
        bpm._name = self._name
        bpm._header = 'bpy_{self._name}.h'.format(self=self),
        bpm._source = 'bpy_{self._name}.cpp'.format(self=self),
        bpm._content = 'BOOST_PYTHON_MODULE({self._name})\n{{\n{content}\n}}\n'.format(self=self, content=content)
        bpm._uids = self._uids

    def get_kwargs(self):
        """This method doesn't return kwargs because creation is carried out inplace"""
        # First, create the content
        content = self.scan_classes(self._project, 1)
        kwargs = {
            'parent': self._container,
            'header': 'bpy_{self._name}.h'.format(self=self),
            'source': 'bpy_{self._name}.cpp'.format(self=self),
            'content': 'BOOST_PYTHON_MODULE({self._name})\n{{\n{content}\n}}\n'.format(self=self, content=content),
            'name': self._name,
            'uids': self._uids
        }
        return kwargs

    def Validate(self):
        """implement OnOK button"""
        self._name = self.m_textCtrl21.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Module name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Module name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    # Handlers for NewBoostPythonModule events.
    def OnCancel(self, event):
        """Handles cancel button"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Handles ok button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-

"""This represents a pending task.
The task uses glass_clock as tree icon"""

import wx
import model
import tran
import app.resources as rc


class Task(model.TComponent):
    """Declares the task element"""
    task_container = True  # allow subtasks

    #visual methods
    @tran.TransactionalMethod('move task {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_task_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0
        new_cont = target.status_container
        if new_cont is not self.status_container:
            new_cont.SetStatus(self)
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    @property
    def status_container(self):
        """return the status container"""
        return self.parent.status_container

    """Declares a task entry"""
    def __init__(self, **kwargs):
        """Constructor"""
        self._status = kwargs.get('status', 'pending')
        self._priority = kwargs.get('priority', 'normal')
        self._taskType = kwargs.get('type', 'CHANGE')
        self._reference = kwargs.get('reference', 'None')
        self._dateCreated = kwargs.get('dateCreated', 'None')
        self._dateBegin = kwargs.get('dateBegin', 'None')
        self._dateEnd = kwargs.get('dateEnd', '')
        super(Task, self).__init__(**kwargs)

    def get_kwargs(self):
        """Get the info about the class"""
        kwargs = {
            'status': self._status,
            'priority': self._priority,
            'type': self._taskType,
            'reference': self._reference,
            'dateCreated': self._dateCreated,
            'dateBegin': self._dateBegin,
            'dateEnd': self._dateEnd
            }
        kwargs.update(super(Task, self).get_kwargs())
        return kwargs

    #backward-compatibility
    def __setstate__(self, d):
        """Add the creation field if missing"""
        if '_dateCreated' not in d:
            d['_dateCreated'] = wx.DateTime.Now().Format('%Y-%m-%d %H:%M:%S')
        self.__dict__ = d

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('glass_clock')



# -*- coding: utf-8 -*-
"""This module defines a custom event for dispatching messages to logger window"""

import wx


class LoggerEvent(wx.PyCommandEvent):
    """Custom event"""
    _type = wx.NewEventType()

    def __init__(self, message, level=wx.LOG_Info, id=wx.ID_ANY):
        """Initializer"""
        self._message = message
        self._level = level
        wx.PyCommandEvent.__init__(self, LoggerEvent._type, id)

    @property
    def level(self):
        """Returns the message level"""
        return self._level

    @property
    def message(self):
        """Returns the message"""
        return self._message

EVT_LOGGER = wx.PyEventBinder(LoggerEvent._type, 1)


"""Subclass of StackFrame, which is generated by wxFormBuilder."""

from activity.files.ui import ui as ui


# Implementing StackFrame
class StackFrame(ui.StackFrame):
    """Aux pane for stack"""
    def __init__(self, parent, handler):
        """Init"""
        self._handler = handler
        super(StackFrame, self).__init__(parent)

    # Handlers for StackFrame events.
    def OnSelectStackFrame(self, event):
        """"""
        # TODO: Implement OnSelectStackFrame
        pass

    def UpdateData(self, current_thread):
        """Update info with the stack"""
        self.m_stackFrame.Clear()
        # update stack frame
        for element in current_thread.stack:
            i = self.m_stackFrame.Append(element[0])
            self.m_stackFrame.SetClientData(i, element)

    def OnActivateStackFrame(self, event):
        """"""
        # TODO: Implement OnActivateStackFrame
        pass



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
import wx
from  ctx import theContext as context
import analytic.astd as astd
import model.file
from WebBrowserPane import WebBrowserPane
import app.resources as rc


class web_browser(wx.PyEvtHandler):
    """Class for providing ast explorer"""
    instance = None

    def __init__(self):
        """Initialize web explorer"""
        super(web_browser, self).__init__()
        #create the menus
        self._menuid = context.frame.new_tool_id()
        self._imenu = wx.MenuItem(context.frame.menuTools, self._menuid, u"Web browser",
            u"show embedded web browser", wx.ITEM_NORMAL)
        context.frame.AddToolsMenu('Web browser', self._imenu)
        #bind the menu handlers
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateWebBrowser, id=self._menuid)
        self.Bind(wx.EVT_MENU, self.OnWebBrowser, id=self._menuid)

    @classmethod
    def load(cls):
        """Setup tool for the environment"""
        return web_browser()

    def OnUpdateWebBrowser(self, event):
        """Handle the update"""
        event.Enable(True)

    def OnWebBrowser(self, event):
        """Handle the command"""
        frame = context.frame
        p = WebBrowserPane(frame.docBook, frame)
        frame.docBook.Freeze()
        frame.docBook.AddPage(p, 'web', True, rc.GetBitmapIndex("info"))
        p.NavigateTo('https://www.google.com/')
        frame.docBook.Thaw()
